@@ -10,7 +10,7 @@ use LWP::UserAgent;
 use URI;
 use URI::Escape qw(uri_unescape);
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 $VERSION = eval $VERSION;
 
 sub new {
@@ -19,30 +19,30 @@ sub new {
 
     my $key = $params{apikey} or croak q('apikey' is required);
 
-    my $self = bless {
-        key => uri_unescape($key),
-    }, $class;
+    my $self = bless \ %params, $class;
+    $self->{key} = uri_unescape($key),
 
     $self->ua(
         $params{ua} || LWP::UserAgent->new(agent => "$class/$VERSION")
     );
 
-    if ($params{debug}) {
+    if ($self->{debug}) {
         my $dump_sub = sub { $_[0]->dump(maxlength => 0); return };
         $self->ua->set_my_handler(request_send  => $dump_sub);
         $self->ua->set_my_handler(response_done => $dump_sub);
     }
-
-    if ($params{https}) {
-        croak q('https' requires Crypt::SSLeay or IO::Socket::SSL)
-            unless eval { require Net::HTTPS; 1 };
-
-        $self->{https} = 1;
+    elsif (exists $self->{compress} ? $self->{compress} : 1) {
+        $self->ua->default_header(accept_encoding => 'gzip,deflate');
     }
 
+    croak q('https' requires LWP::Protocol::https)
+        if $self->{https} and not $self->ua->is_protocol_supported('https');
+
     return $self;
 }
 
+sub response { $_[0]->{response} }
+
 sub ua {
     my ($self, $ua) = @_;
     if ($ua) {
@@ -62,15 +62,15 @@ sub geocode {
 
     my $country = $params{country};
 
-    my $proto = $self->{https} ? 'https' : 'http';
-    my $uri = URI->new("$proto://www.mapquestapi.com/geocoding/v1/address");
+    my $uri = URI->new('http://www.mapquestapi.com/geocoding/v1/address');
     $uri->query_form(
         key      => $self->{key},
         location => $location,
         $country ? (adminArea1 => $country) : (),
     );
+    $uri->scheme('https') if $self->{https};
 
-    my $res = $self->ua->get($uri);
+    my $res = $self->{response} = $self->ua->get($uri);
     return unless $res->is_success;
 
     # Change the content type of the response from 'application/json' so
@@ -80,16 +80,16 @@ sub geocode {
     my $data = eval { from_json($res->decoded_content) };
     return unless $data;
 
-    my @locations = @{ $data->{results}[0]{locations} || [] };
-    if (@locations) {
-        $#locations = 0 unless wantarray;
+    my @results = @{ $data->{results}[0]{locations} || [] };
+    if (@results) {
+        $#results = 0 unless wantarray;
 
         # Keep the location data structure flat.
         my $provided = $data->{results}[0]{providedLocation}{location};
-        $_->{providedLocation} = $provided for @locations;
+        $_->{providedLocation} = $provided for @results;
     }
 
-    return wantarray ? @locations : $locations[0];
+    return wantarray ? @results : $results[0];
 }
 
 sub batch {
@@ -102,14 +102,14 @@ sub batch {
 
     $_ = Encode::encode('utf-8', $_) for @$locations;
 
-    my $proto = $self->{https} ? 'https' : 'http';
-    my $uri = URI->new("$proto://www.mapquestapi.com/geocoding/v1/batch");
+    my $uri = URI->new('http://www.mapquestapi.com/geocoding/v1/batch');
     $uri->query_form(
         key      => $self->{key},
         location => $locations,
     );
+    $uri->scheme('https') if $self->{https};
 
-    my $res = $self->ua->get($uri);
+    my $res = $self->{response} = $self->ua->get($uri);
     return unless $res->is_success;
 
     # Change the content type of the response from 'application/json' so
@@ -161,6 +161,11 @@ Geocoding Web Service.
 =head2 new
 
     $geocoder = Geo::Coder::Mapquest->new(apikey => 'Your API key')
+    $geocoder = Geo::Coder::Mapquest->new(
+        apikey => 'Your API key'
+        https  => 1,
+        debug  => 1,
+    )
 
 Creates a new geocoding object.
 
@@ -178,7 +183,7 @@ object.
     @locations = $geocoder->geocode(location => $location)
 
 In scalar context, this method returns the first location result; and in
-list context it returns all locations results.
+list context it returns all location results.
 
 Each location result is a hashref; a typical example looks like:
 
@@ -213,6 +218,12 @@ Allows up to 100 locations to be geocoded in the same request.  Returns
 a list of results, each of which is a reference to a list of locations.
 Will croak if more than 100 locations are given.
 
+=head2 response
+
+    $response = $geocoder->response()
+
+Returns an L<HTTP::Response> object for the last submitted request. Can be
+used to determine the details of an error.
 
 =head2 ua
 
@@ -240,9 +251,6 @@ service at this time.
 
 L<http://www.mapquestapi.com/geocoding/>
 
-L<Geo::Coder::Bing>, L<Geo::Coder::Google>, L<Geo::Coder::Multimap>,
-L<Geo::Coder::Yahoo>
-
 =head1 REQUESTS AND BUGS
 
 Please report any bugs or feature requests to
@@ -278,13 +286,13 @@ L<http://rt.cpan.org/Public/Dist/Display.html?Name=Geo-Coder-Mapquest>
 
 =item * Search CPAN
 
-L<http://search.cpan.org/dist/Geo-Coder-Mapquest>
+L<http://search.cpan.org/dist/Geo-Coder-Mapquest/>
 
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2010 gray <gray at cpan.org>, all rights reserved.
+Copyright (C) 2009-2011 gray <gray at cpan.org>, all rights reserved.
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
